!-----------------------BEGIN NOTICE -- DO NOT EDIT-----------------------
! NASA Goddard Space Flight Center Land Data Toolkit (LDT) v1.0
!-------------------------END NOTICE -- DO NOT EDIT-----------------------
!BOP
!
! !ROUTINE: read_SiB2_parms
! \label{read_SiB2_parms}
!
! !REVISION HISTORY:
!  02 Dec 2013: KR Arsenault; Initial Specification
!
! !INTERFACE:
subroutine read_SiB2_parms(n, nvegtypes, &
                           z2,     &  ! 1
                           z1,     &  ! 2
                           vcover, &  ! 3
                           chil,   &  ! 4
                           sodep,  &  ! 5
                           rootd,  &  ! 6
                           phc,    &  ! 7
                           tran1,  &  ! 8
                           tran2,  &  ! 9
                           tran3,  &  ! 10
                           tran4,  &  ! 11
                           ref1,   &  ! 12
                           ref2,   &  ! 13
                           ref3,   &  ! 14
                           ref4,   &  ! 15
                           vmax0,  &  ! 16
                           effcon, &  ! 17
                           gradm,  &  ! 18
                           binter, &  ! 19
                           atheta, &  ! 20
                           btheta, &  ! 21
                           trda,   &  ! 22
                           trdm,   &  ! 23
                           trop,   &  ! 24
                           respcp, &  ! 25
                           slti,   &  ! 26
                           shti,   &  ! 27
                           hltii,  &  ! 28
                           hhti,   &  ! 29
                           soref1, &  ! 30
                           soref2, &  ! 31
                           bee,    &  ! 32
                           phsat,  &  ! 33
                           satco,  &  ! 34 
                           poros,  &  ! 35
                           islope, &  ! 36
                           wopt,   &  ! 37
                           wsat,   &  ! 38
                           zm )       ! 39

! !USES:
  use ESMF
  use LDT_coreMod,     only : LDT_rc, LDT_domain
  use LDT_logMod,      only : LDT_logunit, LDT_getNextUnitNumber, &
          LDT_releaseUnitNumber, LDT_endrun
  use SiB2_parmsMod

  implicit none

! !ARGUMENTS: 
  integer,   intent(in) :: n
  integer,   intent(in) :: nvegtypes
  real,   intent(inout) :: z2(LDT_rc%lnc(n),LDT_rc%lnr(n),nvegtypes)
  real,   intent(inout) :: z1(LDT_rc%lnc(n),LDT_rc%lnr(n),nvegtypes)
  real,   intent(inout) :: vcover(LDT_rc%lnc(n),LDT_rc%lnr(n),nvegtypes)
  real,   intent(inout) :: chil(LDT_rc%lnc(n),LDT_rc%lnr(n),nvegtypes)
  real,   intent(inout) :: sodep(LDT_rc%lnc(n),LDT_rc%lnr(n),nvegtypes)
  real,   intent(inout) :: rootd(LDT_rc%lnc(n),LDT_rc%lnr(n),nvegtypes)
  real,   intent(inout) :: phc(LDT_rc%lnc(n),LDT_rc%lnr(n),nvegtypes)
  real,   intent(inout) :: tran1(LDT_rc%lnc(n),LDT_rc%lnr(n),nvegtypes)
  real,   intent(inout) :: tran2(LDT_rc%lnc(n),LDT_rc%lnr(n),nvegtypes)
  real,   intent(inout) :: tran3(LDT_rc%lnc(n),LDT_rc%lnr(n),nvegtypes)
  real,   intent(inout) :: tran4(LDT_rc%lnc(n),LDT_rc%lnr(n),nvegtypes)
  real,   intent(inout) :: ref1(LDT_rc%lnc(n),LDT_rc%lnr(n),nvegtypes)
  real,   intent(inout) :: ref2(LDT_rc%lnc(n),LDT_rc%lnr(n),nvegtypes)
  real,   intent(inout) :: ref3(LDT_rc%lnc(n),LDT_rc%lnr(n),nvegtypes)
  real,   intent(inout) :: ref4(LDT_rc%lnc(n),LDT_rc%lnr(n),nvegtypes)
  real,   intent(inout) :: vmax0(LDT_rc%lnc(n),LDT_rc%lnr(n),nvegtypes)
  real,   intent(inout) :: effcon(LDT_rc%lnc(n),LDT_rc%lnr(n),nvegtypes)
  real,   intent(inout) :: gradm(LDT_rc%lnc(n),LDT_rc%lnr(n),nvegtypes)
  real,   intent(inout) :: binter(LDT_rc%lnc(n),LDT_rc%lnr(n),nvegtypes)
  real,   intent(inout) :: atheta(LDT_rc%lnc(n),LDT_rc%lnr(n),nvegtypes)
  real,   intent(inout) :: btheta(LDT_rc%lnc(n),LDT_rc%lnr(n),nvegtypes)
  real,   intent(inout) :: trda(LDT_rc%lnc(n),LDT_rc%lnr(n),nvegtypes)
  real,   intent(inout) :: trdm(LDT_rc%lnc(n),LDT_rc%lnr(n),nvegtypes)
  real,   intent(inout) :: trop(LDT_rc%lnc(n),LDT_rc%lnr(n),nvegtypes)
  real,   intent(inout) :: respcp(LDT_rc%lnc(n),LDT_rc%lnr(n),nvegtypes)
  real,   intent(inout) :: slti(LDT_rc%lnc(n),LDT_rc%lnr(n),nvegtypes)
  real,   intent(inout) :: shti(LDT_rc%lnc(n),LDT_rc%lnr(n),nvegtypes)
  real,   intent(inout) :: hltii(LDT_rc%lnc(n),LDT_rc%lnr(n),nvegtypes)
  real,   intent(inout) :: hhti(LDT_rc%lnc(n),LDT_rc%lnr(n),nvegtypes)
  real,   intent(inout) :: soref1(LDT_rc%lnc(n),LDT_rc%lnr(n),nvegtypes)
  real,   intent(inout) :: soref2(LDT_rc%lnc(n),LDT_rc%lnr(n),nvegtypes)
  real,   intent(inout) :: bee(LDT_rc%lnc(n),LDT_rc%lnr(n),nvegtypes)
  real,   intent(inout) :: phsat(LDT_rc%lnc(n),LDT_rc%lnr(n),nvegtypes)
  real,   intent(inout) :: satco(LDT_rc%lnc(n),LDT_rc%lnr(n),nvegtypes)
  real,   intent(inout) :: poros(LDT_rc%lnc(n),LDT_rc%lnr(n),nvegtypes)
  real,   intent(inout) :: islope(LDT_rc%lnc(n),LDT_rc%lnr(n),nvegtypes)
  real,   intent(inout) :: wopt(LDT_rc%lnc(n),LDT_rc%lnr(n),nvegtypes)
  real,   intent(inout) :: wsat(LDT_rc%lnc(n),LDT_rc%lnr(n),nvegtypes)
  real,   intent(inout) :: zm(LDT_rc%lnc(n),LDT_rc%lnr(n),nvegtypes)

! !DESCRIPTION:
!  This subroutine retrieves the greenness fraction climatology for the 
!  specified month and returns the values in the latlon projection
!  
!  The arguments are:
!  \begin{description}
!  \item[n]
!   index of the nest
!  \item[array]
!   output field with the retrieved greenness fraction
!  \end{description}
!
!EOP      
  integer        :: c,r,v
  integer        :: ftn
  logical        :: file_exists
  character(140) :: vegfile
  character(2)   :: numvegchar
! _____________________________________________________

! Check for SiB2 file directory:
!  inquire(file=trim(LDT_rc%sib2parmsdir(n)), exist=file_exists)
!  if(.not.file_exists) then 
!     write(LDT_logunit,*) "SiB2 Directory: ",trim(LDT_rc%sib2parmsdir(n))," not found."
!     write(LDT_logunit,*) "Program stopping ..."
!     call LDT_endrun
!  endif

! Loop over the ISA/SiB2 veg classes (1 file per class):
  do v=1,nvegtypes

     write( numvegchar, '(i2.2)' ) v 
     vegfile = trim(SiB2_struc(n)%sib2parmsdir)//"/mapper_"//numvegchar//"_inv"

     inquire(file=trim(vegfile), exist=file_exists)
     if( file_exists ) then 

       write(LDT_logunit,*) "Opening SiB2 file: ",trim(vegfile)
       ftn = LDT_getNextUnitNumber()
!       open(ftn, file=trim(vegfile), access='direct',status='old', &
!            form="unformatted", recl=4)
  
! READ DATA ...

       do r=1,LDT_rc%lnr(n)
          do c=1,LDT_rc%lnc(n)
!            if(array1(c,r,v).lt.0) then
!               array1(c,r,v) = LDT_rc%udef
               z2(c,r,v) = 2.
!            endif
          enddo
       enddo

       call LDT_releaseUnitNumber(ftn)

  ! If file is missing:
    else 
       write(LDT_logunit,*) "SiB2 file: ",trim(vegfile)," not found."
       write(LDT_logunit,*) " File not read in for veg type :: ",v 
!       write(LDT_logunit,*) "Program stopping ..."
!       call LDT_endrun
     endif

  enddo  ! End veg type loop

end subroutine read_SiB2_parms
