!-----------------------BEGIN NOTICE -- DO NOT EDIT-----------------------
! NASA Goddard Space Flight Center Land Data Toolkit (LDT) v1.0
!-------------------------END NOTICE -- DO NOT EDIT-----------------------
#include "LDT_misc.h"
module VIC_parmsMod
!BOP
!
! !MODULE: VIC_parmsMod
!
! !DESCRIPTION:
!  The code in this file implements routines to read VIC parameter
!  data. 
!  \subsubsection{Overview}
!  This routines in this module provides routines to read the 
!  VIC parameter file data.
!
! !REVISION HISTORY:
!
!  04 Nov 2013: K. Arsenault: Added layers for VIC model
!
#if(defined USE_NETCDF3 || defined USE_NETCDF4)
  use netcdf
#endif

  use ESMF
  use LDT_coreMod
  use LDT_historyMod
  use LDT_paramDataMod
  use LDT_logMod
  use LDT_paramMaskCheckMod

  implicit none

  PRIVATE
!------------------------------------------------------------------------------
! !PUBLIC MEMBER FUNCTIONS:
!------------------------------------------------------------------------------
  public :: VICparms_init    !allocates memory for required structures
  public :: VICparms_writeHeader
  public :: VICparms_writeData

!------------------------------------------------------------------------------
! !PUBLIC TYPES:
!------------------------------------------------------------------------------
  public :: VIC_struc

  type, public :: vic_type_dec

     real      :: vic_undef

! -  VIC-specific:
     type(LDT_paramEntry) :: vic  ! VIC parameters (collective)

  end type vic_type_dec

  type(vic_type_dec), allocatable :: VIC_struc(:)

contains

!BOP
! 
! !ROUTINE: VICparms_init
! \label{VICparms_init}
! 
! !INTERFACE:
  subroutine VICparms_init

! !USES:
   use LDT_logMod,    only : LDT_verify, LDT_endrun, &
             LDT_getNextUnitNumber, LDT_releaseUnitNumber
!
! !DESCRIPTION:
!
! Variable Infiltration Capacity (VIC) model parameters.
! 
!  The routines invoked are: 
!  \begin{description}
!   \item[vicParmssetup](\ref{vicParmssetup}) \newline
!    calls the registry to invoke the vicParms setup methods. 
!  \end{description}
!
!EOP
   implicit none
   integer  :: n
   integer  :: c,r,m,k
   integer  :: rc
   integer  :: file_status
   logical  :: file_exists
   logical  :: vic_select 

   ! _____________________________________________________________________

   allocate(VIC_struc(LDT_rc%nnest))

   vic_select = .false.
   do n=1,LDT_rc%nnest
      ! - VIC parameters:
      call set_param_attribs(VIC_struc(n)%vic,"VIC")
      if( VIC_struc(n)%vic%selectOpt == 1 ) then
         vic_select = .true.
      endif
   enddo

   if( vic_select ) then
     write(LDT_logunit,*)" - - - - - - - - - - VIC LSM Parameters - - - - - - - - - - - - -"

     write(LDT_logunit,*)" ** MSG:  NO VIC Model Parameters available yet, other than"
     write(LDT_logunit,*)" ** MSG:  specific landcover and landmask."
      
   endif


 end subroutine VICparms_init


 subroutine VICparms_writeHeader(n,ftn,dimID,monthID)
   
   integer   :: n 
   integer   :: ftn
   integer   :: dimID(3)
   integer   :: monthID
   
   integer   :: t_dimID(3)
   
   if( VIC_struc(n)%vic%selectOpt == 1 ) then

   endif
   
   
 end subroutine VICparms_writeHeader
 
  subroutine VICparms_writeData(n,ftn)

    integer   :: n 
    integer   :: ftn

    if( VIC_struc(n)%vic%selectOpt == 1 ) then
!      call VICparms_writeData(n,ftn)
!      call VICparms_writeData(n,ftn)
    endif

  end subroutine VICparms_writeData

!BOP
! !ROUTINE:  set_param_attribs
! \label{set_param_attribs}
!
! !INTERFACE:
  subroutine set_param_attribs(paramEntry, short_name)

! !DESCRIPTION:
!   This routine reads over the parameter attribute entries
!   in the param_attribs.txt file.
!
! !USES:
   type(LDT_paramEntry),intent(inout) :: paramEntry
   character(len=*),    intent(in)    :: short_name

! ____________________________________________________
    
   
   paramEntry%short_name = trim(short_name)
   paramEntry%vlevels = 1
   paramEntry%selectOpt = 1
   paramEntry%source = "VIC"
   paramEntry%units ="none"
   paramEntry%num_times = 1
   paramEntry%num_bins = 1
   paramEntry%standard_name = trim(short_name)

  end subroutine set_param_attribs

end module VIC_parmsMod

