!-----------------------BEGIN NOTICE -- DO NOT EDIT-----------------------
! NASA Goddard Space Flight Center Land Data Toolkit (LDT) v1.0
!-------------------------END NOTICE -- DO NOT EDIT-----------------------
#include "LDT_misc.h"
!BOP
!
! !ROUTINE: read_regmask_gis
! \label{read_regmask_gis}
!
! !REVISION HISTORY:
!  03 Sep  2012: KR Arsenault; Read in Regional Mask File
!
! !INTERFACE:
subroutine read_regmask_gis( n, regmask )

! !USES:
  use LDT_coreMod,     only : LDT_rc
  use LDT_logMod,      only : LDT_logunit, LDT_getNextUnitNumber, &
       LDT_releaseUnitNumber, LDT_endrun
  use LDT_fileIOMod,   only : readLISdata, LDT_checkDomainExtents
  use LDT_gridmappingMod

  implicit none
! !ARGUMENTS: 
  integer, intent(in)  :: n          ! nest index
  real,    intent(out) :: regmask(LDT_rc%lnc(n),LDT_rc%lnr(n),1)

! !DESCRIPTION:
!  This subroutine retrieves regional mask information for clipping
!   out designated area.
!
!  The arguments are:
!  \begin{description}
!  \item[n]
 !   index of the nest
!  \item[regmask]
 !   output field with regional mask information
!  \end{description}
!EOP
   integer :: ftn
   logical :: file_exists
   integer :: i, t, c, r
!________________________________________________________________________

   regmask = 0.

   write(LDT_logunit,*) "MSG: Reading Regional Mask Info File: ",&
        trim(LDT_rc%regfile(n))

!- Check extents of input file grid:
   call LDT_checkDomainExtents(n,LDT_rc%reg_gridDesc(n,:))

!- Check file and open:
   inquire(file=trim(LDT_rc%regfile(n)), exist=file_exists)
   if(.not.file_exists) then 
      write(LDT_logunit,*) "Regional mask map ",trim(LDT_rc%regfile(n))," not found."
      write(LDT_logunit,*) "Program stopping ..."
      call LDT_endrun
   endif

   ftn = LDT_getNextUnitNumber()
   open(ftn,file=LDT_rc%regfile(n),form='unformatted',status='old',&
            access='direct',recl=4)

!- Read and aggregate data:
   select case ( LDT_rc%reg_gridtransform(n) )

    case( "none", "mode", "neighbor" )
      call readLISdata(n, ftn, LDT_rc%reg_proj, LDT_rc%reg_gridtransform(n), &
                      LDT_rc%reg_gridDesc(n,:), 1, regmask)  ! 1 indicates 2D layer

    case default
      write(LDT_logunit,*) ' This spatial transform for regional mask not available ... '
      write(LDT_logunit,*) 'Program stopping ...'
      call LDT_endrun
   end select
! _____________________________________________

   call LDT_releaseUnitNumber(ftn)

   write(LDT_logunit,*) "MSG: Done reading regional mask files"

 end subroutine read_regmask_gis
